-------------------------------------------------------------------------------
-- Multiplexer
--
-- Generics:
--   - n : depth of the multiplexer tree (number of address bits)
--
-- Ports:
--   - i [in]  : input signals
--   - a [in]  : address
--   - o [out] : multiplexed signal
--
-- Recursive binary tree structure.
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity mux is
  generic ( n : positive );
  port ( i : in  std_logic_vector(2**n-1 downto 0);
         a : in  std_logic_vector(n-1 downto 0);
         o : out std_logic );
end entity;

architecture arch of mux is
  component mux is
    generic ( n : positive );
    port ( i : in  std_logic_vector(2**n-1 downto 0);
           a : in  std_logic_vector(n-1 downto 0);
           o : out std_logic );
  end component;

  signal o0 : std_logic;
  signal o1 : std_logic;
begin -------------------------------------------------------------- Final step

  single : if n = 1 generate
    o0 <= i(0);
    o1 <= i(1);
  end generate;

  -------------------------------------------------------------- Recursive step

  recursive : if n > 1 generate
    mux0 : mux
      generic map ( n => n-1 )
      port map ( i => i(2**(n-1)-1 downto 0),
                 a => a(n-2 downto 0),
                 o => o0 );
    mux1 : mux
      generic map ( n => n-1 )
      port map ( i => i(2**n-1 downto 2**(n-1)),
                 a => a(n-2 downto 0),
                 o => o1 );
  end generate;

  ---------------------------------------------------------- 2-to-1 multiplexer

  o <= o0 when a(n-1) = '0' else
       o1;

end architecture; -------------------------------------------------------------
