-------------------------------------------------------------------------------
-- Right shift operator
--
-- Generics:
--   - w : width of the input operand
--   - n : number of shifting stages (width of the shift amount signal s)
--
-- Ports:
--   - i [in]  : input signal
--   - s [in]  : shift amount
--   - o [out] : right-shifted signal
--
-- Recursive structure with n stages.
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity right_shift is
  generic ( w : positive;
            n : positive );
  port ( i : in  std_logic_vector(w-1 downto 0);
         s : in  std_logic_vector(n-1 downto 0);
         o : out std_logic_vector(w+2**n-2 downto 0) );
end entity;

architecture arch of right_shift is
  component right_shift is
    generic ( w : positive;
              n : positive );
    port ( i : in  std_logic_vector(w-1 downto 0);
           s : in  std_logic_vector(n-1 downto 0);
           o : out std_logic_vector(w+2**n-2 downto 0) );
  end component;

  signal o0 : std_logic_vector(w+2**(n-1)-1 downto 0);
begin ------------------------------------------------------- 2^(n-1)-bit shift

  o0 <= i & (2**(n-1)-1 downto 0 => '0') when s(n-1) = '0' else
        (2**(n-1)-1 downto 0 => '0') & i;
  
  ----------------------------------------------------------------- Final stage

  single : if n = 1 generate
    o <= o0;
  end generate;

  ------------------------------------------------------------- Recursive stage

  recursive : if n > 1 generate
    shift : right_shift
      generic map ( w => w+2**(n-1),
                    n => n-1 )
      port map ( i => o0,
                 s => s(n-2 downto 0),
                 o => o );
  end generate;

end architecture; -------------------------------------------------------------
