-------------------------------------------------------------------------------
-- Common utility package for floating-point components
--
-- Components:
--   - fp_round  : final rounding step for floating-point exponent and mantissa
--   - fp_format : creates a FP representation of a number from its sign,
--                 exponent and mantissa
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

package pkg_fp_common is

  -----------------------------------------------------------------------------
  -- Final rounding step for floating-point exponent and mantissa
  -----------------------------------------------------------------------------

  component fp_round is
    generic ( wE : positive;
              wF : positive );
    port ( eX : in  std_logic_vector(wE downto 0);
           mX : in  std_logic_vector(wF+2 downto 0);
           eR : out std_logic_vector(wE downto 0);
           mR : out std_logic_vector(wF downto 0) );
  end component;

  -----------------------------------------------------------------------------
  -- Creates a FP representation of a number from its sign, exponent and
  -- mantissa
  -----------------------------------------------------------------------------

  component fp_format is
    generic ( wE : positive;
              wF : positive );
    port ( s  : in  std_logic;
           e  : in  std_logic_vector(wE downto 0);
           m  : in  std_logic_vector(wF downto 0);
           fp : out std_logic_vector(wE+wF+2 downto 0) );
  end component;

end package;
